# Title: biomonitoR: an R package for managing ecological data and calculating biomonitoring indices
# Authors: Alex Laini, Simone Guareschi, Rossano Bolpagni, Gemma Burgazzi, Daniel Bruno, Cayetano Gutierrez-Canovas,
# Rafael Miranda, Cedric Mondy, Gabor Varbiro, Tommaso Cancellario

# This is an example code to run analyses of community data with the biomonitoR package.
# For any question or bug reporting please drop a mail to alex.laini@unito.it
# The following topics will be covered:
#   1. INSTALLATION
#   2. DATA IMPORT AND MANIPULATION
#   3. CALCULATION OF RICHNESS, DIVERSITY AND BIOMONITORING INDICES
#   4. FUNCTIONAL INDICES CALCULATION
#   5. ANALYSIS ON OTHER BIOTIC GROUPS
#   6. MANAGE TRAIT INFORMATION
#   7. USER'S OWN TRAITS DATASET
#     a. Traits of North-American aquatic insects
#     b. Dispersal traits of aquatic macroinvertebrates

# Imports: hunspell, dplyr, tidyr, FD, geometry, ade4, tibble, ape, lifecycle,
# plotly, RColorBrewer, rjson

###############################
####### 1. INSTALLATION #######
###############################

# biomonitoR is currently available on github.
# To install it you can use the function install_github from the devtools package.
# Please install it if missing from your library.

library(devtools)
install_github("alexology/biomonitoR", ref = "main", build_vignettes = TRUE)


###############################################
####### 2. DATA IMPORT AND MANIPULATION #######
###############################################


####### DATA IMPORT #######

# Load packages, we will use the vegan and ade4 packages for some examples. Please install them if missing from your library.
library(biomonitoR)
library(vegan)
library(ade4)
library(StatMatch)
library(ggplot2)
library(readxl)

# Load the built-in macroinvertebrate dataset. Although we will work with macroinvertebrates, any biotic community can be used.
data(mi_prin)

# Take a look to the data; taxa are reported on the first column and abundances for each site on the other columns
# taxonomic resolution is mainly at genus/family.
mi_prin[1:5, 1:5]


# Import data in the biomonitoR format.
# At first, we use the as_biomonitor function.
# This function merge the user dataset with the reference taxonomic dataset.
# The user can use the 4 reference datasets available in biomonitoR: diatoms, macrophytes,
# macroinvertebrates and fish. You can select one of these datasets with the argument group.
data_asb <- as_biomonitor(mi_prin, group = "mi")

# As you can see by printing the object the taxonomic information has been added to
# the user dataset. This information is added independently of the input taxonomic level (e.g. family, genus).
data_asb

# We can plot the object and see the general structure of the dataset. The plot is interactive, try it!
plot(data_asb, parent = "Order", child = "Family")

# The plot shows the families in the dataset divided by order. You can change the
# taxonomic level to show by setting the arguments parent and child. You can see that some
# families do not have an order (e.g. Mollusca experts usually avoid to group taxa into orders)
# and an order has not families (because organisms were identified at the order level).
# You can set other taxonomic levels.
plot(data_asb, parent = "Phylum", child = "Family")

# You can use abundance instead of presence-absence data. In this case, the abundance
# of each taxon is obtained by summing all the abundances in the dataset.
plot(data_asb, parent = "Order", child = "Family", type = "abundance")

# You can transform the abundances to have a more readable output;
plot(data_asb, parent = "Order", child = "Family", type = "abundance", trans = log1p)

# or you can plot the frequency, the number of samples in which a taxon have been found
# over the total number of samples.
plot(data_asb, parent = "Order", child = "Family", type = "frequency")

####### INCORRECT TAXA NAMES #######

# The as_biomonitor function check if the taxa names in the user database are consistent
# with the names of the reference dataset. If missing, the non-identified taxon will be excluded and
# a warning will be printed. Let's try an example
# At first, generate an error in the dataset.
mi_prin_fake <- mi_prin
mi_prin_fake$Taxa <- gsub("Baetis", "Betiss", mi_prin_fake$Taxa)

# use as_biomonitor, that will print a warning
# The taxon with the erroneous name has been excluded to ensure
# consistency among taxonomic level. Remember that one of the main
# goals in biomonitoR is to allow calculation at multiple taxonomic levels.
# If the taxonomic information is missing we cannot work at multiple taxonomic levels.
data_asb_fake <- as_biomonitor(mi_prin_fake, group = "mi")

# To see which taxa have been excluded use the traceB argument.
data_asb_fake <- as_biomonitor(mi_prin_fake, group = "mi", traceB = TRUE)

# Basically the result is a list. The first object of the list is the user dataset merged with
# the reference dataset that we already seen and will not be explored further.
# The second object summarize automatic or user-defined changes that have been
# done to the dataset. We will explore this option later in detail.
# The third object list the wrong taxa names and suggest alternative names that are
# present in the reference dataset.

# The correct name has been suggested.
data_asb_fake[[3]]

# Now the user can fix this problem by modifying her/his dataset
# or use the argument to_change in as_biomonitoR.
# We build the data.frame for doing such change.
to_change <- data_asb_fake[[3]]
names(to_change) <- c("Taxon", "Correct_Taxon")

# We add another change, that for macroinvertebrates is done by default
# but we need to insert it manually when using the to_change argument.
# biomonitoR change by default Hydracarina and Acariformes in the more acceptable
# Trombidiformes.
to_change[2,] <- c("Hydracarina", "Trombidiformes")

# Now the problem is fixed.
data_asb_fake <- as_biomonitor(mi_prin_fake, group = "mi", traceB = TRUE, to_change = to_change)

####### SUBSET THE RESULTS OF as_biomonitor #######

# We can take a subset of the imported data.
# For instance, we can decide to work within an insect order.
data_asb_ephemeroptera <- subset(data_asb, taxa = "Ephemeroptera")

# By printing the object you can see that only the taxa within the order Ephemeroptera have been retained.
data_asb_ephemeroptera

# We can plot the result of the subset and see that only one order is present;
plot(data_asb_ephemeroptera, type = "abundance")

# or we can work with all the insects.
data_asb <- subset(data_asb, taxa = "Insecta")

# and again, we can plot the results
# presence absence data.
plot(data_asb)

# Abundance data.
plot(data_asb, type = "abundance")

# Frequency data, the number of samples in which a taxon has been detected.
plot(data_asb, type = "frequency")

# You can subset data at multiple taxonomic levels, here an example at order (Ephemeroptera) and
# family (Chironomidae) level.
data_asb_ephemeroptera_chironomidae <- subset(data_asb, taxa = c("Ephemeroptera", "Chironomidae"))

# Plot it.
plot(data_asb_ephemeroptera_chironomidae)

####### REFERENCE DATASET #######

# biomonitoR comes with 4 default reference dataset for diatoms, macrophytes, macroinvertebrates and fish.
# If you are not satisfied with these dataset or they are not suitable for your biotic group
# you can build your own database starting from a taxonomic tree.

# Here an example.
# Load example data. The object Tree is an example of taxonomic tree based on aquatic invertebrates.
data(Tree)
Tree

# Tree is a standard taxonomic tree that needs to be transformed into a reference database suitable for biomonitoR.
# We can use the function ref_from_tree.
ref_custom <- ref_from_tree(Tree)

# Some functions uses online database to search for taxonomic information.
# Let's try with GBIF. The input is a vector of taxa, usually the first column in the user dataset.
ref_gbif <- get_gbif_taxa_tree(mi_prin$Taxa)

# The result is a list, the first object being the taxonomic tree inferred from GBIF.
ref_gbif[[1]]

# While the second lists all the taxa that have not been found by querying GBIF.
ref_gbif[[2]]

# Unfortunately it is not possible to correct misspelled names with this function.
get_gbif_taxa_tree(mi_prin_fake$Taxa)


####### aggregate_taxa #######

# When you are satisfied with the as_biomonitor outputs, you must use the
# aggregate_taxa function that aggregates the as_biomonitor object at different taxonomic levels.
data_agg <- aggregate_taxa(data_asb)

# The data_agg object is basically a list and you can access a data.frame for each taxonomic level.
data_agg[["Phylum"]]

# Some individuals are unassigned, for example when a taxon is inputted at family level and we
# extract the genus level information.
head(data_agg[["Genus"]])

# You can also plot the aggregate_taxa object. Do this when you have a low number of observations.
# The plot is a barplot reporting the relative abundance of each taxon for each observation.
# Observations are ordered with a cluster analysis to show similarities among them.
plot(data_agg)

# It is also possible to plot indicator taxa. Indicator Taxa Analysis
# (IndVal; Dufrene and Legendre, 1997) is a method to find indicator species and species
# assemblages characterizing groups of sites. The function plot_indicator_taxa implements
# the group-equalized IndVal for both presence-absence and abundance data
# (De Caceres and Legendre, 2009). Please consider that plot_indicator_taxa is intended
# to visualize the main trends present in the data. For having more control on the IndVal
# calculation also consider the indicspecies package (De Caceres and Legendre, 2009).
plot_indicator_taxa(data_agg, thresh = 70, opacity = 0.1)

# It is also possible to plot indicator taxa/taxonomic groups against specified clusters or predefined groups of the samples.
cluster <- as.data.frame(names(mi_prin[-1]))
cluster$cluster <- c("high","high", "high" ,"high", "high", "high", "high", "high", "low",  "low",  "low",  "low" , "low" , "low" , "low"  ,"low" ,"low", 
                     "low",  "low" , "low" , "low" , "low",  "low" , "low" , "low" , "low" , "low" , "low",  "low" , "high", "high" ,"high", "high", "high",
                     "high" ,"high" ,"high", "high", "high" ,"high")

names(cluster) <- c("sites","clusters")

plot_indicator_taxa(data_agg, cluster=cluster,thresh = 40, opacity = 0.1,parent = "Order")

######################################
####### 3. CALCULATING INDICES #######
######################################

####### abundance and richness #######

# Calculating abundance and richness at different taxonomic levels.
# The taxonomic level can be set with the argument tax_lev.
richness(data_agg, tax_lev = "Taxa")

# A warning is printed when unassigned taxa are present at the desired taxonomic level.
richness(data_agg, tax_lev = "Genus")

# Calculate richness at different taxonomic levels at once.
allrich(data_agg)


####### diversity indices #######

# A function for each index implemented in biomonitoR is provided,
# for example for calculating the Shannon index at taxa and family level.
shannon(data_agg, tax_lev = "Taxa")
shannon(data_agg, tax_lev = "Family")

# Calculating all the diversity indices at once, setting the taxonomic level.
allindices(data_agg, tax_lev = "Taxa")


####### biomonitoring indices #######

# biomonitoR include several functions for calculating biomonitoring indices, especially for macroinvertebrates.

# Calculating the Average Score Per Taxon;
aspt(data_agg)

# or the life index with the composite family approach.
life(data_agg, agg = TRUE)

# You can inspect the scores used to calculate the index.
show_scores("life", "extence")


####### biocontamination index #######

# Biocontamination index is an important tool for assessing the impact
# of alien species on native communities.
# Let us assume that the Baetis is an alien genus. We can calculate the bioco index.
# We must specify the reference dataset used for importing data to perform this calculation.
res_bioco <- bioco(data_agg, alien = "Baetis", dfref = "mi")

# This function returns 3 indices: 
# abundance contamination index (ACI);
# richness contamination index (RCI);
# site-specific biocontamination index (SBCI).
head(res_bioco)


#####################################
####### 4. FUNCTIONAL INDICES #######
#####################################

####### assign traits #######

# One of the main challenges in using traits is to merge the taxonomic dataset with the traits dataset
# hen mixed taxonomic resolution is present. Let us assume that traits are specified at genus level
# and the taxonomic dataset at family level, we cannot perform a simple merge.
# We need to search in the taxonomic tree and genera of the traits dataset to the right family.

# We provide an example with the Tachet et al. (2010) dataset, a fuzzy-coded trait dataset
# for macroinvertebrates that is available in biomonitoR. We can use the function assign_traits
# to assign traits to taxa.
data_ts <- assign_traits(data_agg, tax_lev = "Taxa")

# Take a look to a subset of this dataset.
# The user dataset included the family Athericidae
# while the trait information was at genus (Atherix) and species (Atrichops crassipes) level.
# assign_traits uses the information in the taxonomic tree to correctly assign the trait information
# to the user dataset.
# From this subset you can see that the user taxon (Athericidae, column 1) was at family level
# (column 2) and that traits were available for two taxa (column 3) at genus and species level
# (column 4).The taxonomic distance along the biomonitoR reference dataset (column 5) is 3, because we
# need three steps from genus to family (genus -> tribus -> subfamily -> family) and 4
# from species to family (species -> genus -> tribus -> subfamily -> family). The 6th
# column is the first trait.
data_ts[1:2, 1:6]


# Data can be assigned up to family level. For example the user cannot assign traits
# at order level.
assign_traits(data_agg, tax_lev = "Order")


# For calculating functional indices we can use the average trait value for each taxon.
# This is obtained with the average_traits function.
# More control over the selection of taxa for the averaging step can be obtained with
# the function manage_traits (not shown here).
data_ts_av <- average_traits(data_ts)

# Each taxon has now only one row and the assignment information is disappeared.
data_ts_av[1:10, 1:5]

# Now we can calculate functional indices.
# Because we are using fuzzy coded traits we need to set the number of categories for each trait.
# For fuzzy trait management biomonitoR relies on the ade4 package.
col_blocks <- c(8, 7, 3, 9, 4, 3, 6, 2, 5, 3, 9, 8, 8, 5, 7, 5, 4, 4, 2, 3, 8)

# biomonitoR can calculate functional indices using trait stored in a data.frame.
# Let's try to calculate functional dispersion using fuzzy data. We need to specify several arguments.
f_disp(data_agg, trait_db = data_ts_av, nbdim = 2, type = "F", col_blocks = col_blocks)

# This function print a message because the distance matrix generated within this function
# is not euclidean and you can decide to correct this behavior by setting the correction
# argument, for example:
f_disp(data_agg, trait_db = data_ts_av, nbdim = 2, type = "F", col_blocks = col_blocks, correction = "cailliez")

# A finer control over the calculation can be obtained by providing
# a species by species distance matrix of class dist.

# Create the distance matrix with ade4 functions.
rownames(data_ts_av) <- data_ts_av$Taxa
traits_prep <- prep.fuzzy(data_ts_av[, -1], col.blocks = col_blocks)
traits_dist <- ktab.list.df(list(traits_prep))
traits_dist <- dist.ktab(traits_dist, type = "F")

# Functional dispersion.
f_disp(data_agg, trait_db = traits_dist)

# When using a species by species distance matrix we can evaluate the quality of the functional space.
# This is particularly useful when using indices that require a functional space.
# In this case we use the Legendre method, that range from 0 (worst) to 1 (best).
# We can set a threshold ranging from 0 to 1 and the function returns
# the number of dimensions needed to have values greater than such threshold.
# This function also returns the information about the distance matrix being euclidean.
select_pcoa_axes(traits_dist, method = "legendre", tresh = 0.2)



##################################################
####### 5. ANALYSIS ON OTHER BIOTIC GROUPS #######
##################################################

# Create an example dataset.
ex_data <- data.frame(Taxa = c("Homo sapiens", "Mus musculus", "Populus alba", "Gorilla gorilla"),
                      site_1 = sample(1:100, 4),
                      site_2 = sample(1:100, 4))

# Get the taxonomic tree.
tax_tree <- get_gbif_taxa_tree(ex_data$Taxa)

# Import data into biomonitoR format.
ex_asb <- as_biomonitor(ex_data, dfref = tax_tree$taxonomy)

# Plot the results to seethe structure of the dataset.
plot(ex_asb)
plot(ex_asb, type = "abundance")

# Plot species instead of families.
plot(ex_asb, type = "abundance", parent = "Order", child = "Species")

# Subset Mammalia only.
mammalia_asb <- subset(ex_asb, taxa = "Mammalia")

# Use the mammalian dataset for the next step aka aggregating data.
mammalia_agg <- aggregate_taxa(mammalia_asb)

# See the relative abundance of the mammalian species at the investigated sites.
# Not very useful here.
plot(mammalia_agg)

# Calculate species richness and family level.
richness(mammalia_agg, tax_lev = "Species")
richness(mammalia_agg, tax_lev = "Family")

# Calculate the shannon index at species and family level.
shannon(mammalia_agg, tax_lev = "Species")
shannon(mammalia_agg, tax_lev = "Family")



#######################
### 6 MANAGE TRAITS ###
#######################


# Create a toy example to show how some functions for trait management works.
# Select some mayflies name and then retrieve taxonomic information.
example_taxa <- c("Baetis alpinus", "Baetis rhodani", "Centroptilum luteolum")

# Retrieve taxonomic information.
example_taxo_tree <- get_gbif_taxa_tree(example_taxa)

# Transform the taxonomic tree to a reference dataset suitable to import data in biomonitoR.
example_df_ref <- ref_from_tree(example_taxo_tree$taxonomy)

# Create a taxonomic dataset: we will hypothesize two sample and we will randomly assign abundance for 
# each taxon in example taxa.
example_dataset <- data.frame(Taxa = example_taxa, site_1 = rpois(3, lambda = 5), site_2 = rpois(3, lambda = 10))

# Import data into biomonitoR.
example_asb <- as_biomonitor(example_dataset, dfref = example_df_ref)

# Plot the results.
plot(example_asb, parent = "Family", child = "Species", type = "abundance")

# Use aggregate_taxa.
example_agr <- aggregate_taxa(example_asb)

# Create a trait dataset, similarly to what was done with example dataset.
# We will simulate 2 continuous traits.
# We will also simulate two genus level traits.
# Genus level traits are not common but can be present for non-charsimatic taxa (e.g., invertebrates).
example_traits <- data.frame(Taxa = c(example_taxa, "Baetis", "Centroptilum"), trait_1 = c(1:3, 6, 6), trait_2 = c(4:2, 1, 1))
example_traits

# Assign traits to the example dataset, at species level.
# You can see that each taxon has been correctly assigned a trait at species level.
assign_traits(example_agr, example_traits, dfref = example_df_ref)

# Let's try at family level.
# Here we don't have an exact match and both species and genus level information have
# been assigned to family.
example_genus_traits <- assign_traits(example_agr, example_traits, dfref = example_df_ref, tax_lev = "Family")
example_genus_traits

# We can manage this information in two ways.
# 1. We can take the average value.
average_traits(example_genus_traits, type = "C")

# 2. We can take a random subset (compare two runs and see hoe the trait value change).
sample_traits(example_genus_traits)
sample_traits(example_genus_traits)

# A finer control over traits can be used with manage traits.
# This function select taxa that are nearer in the taxonomic tree,
# in this case the genus level information will be selected.
manage_traits(example_genus_traits)

# The selection obtained with the last step tell us that the two genera have the same trait values.
# This can represent ecological equivalence.
# However, this situation can cause problems to the calculation of some functional indices (e.g. functional richness)
# that rely on multivariate functional space.
# One method to solve this issue is to add a small amount of bias to make them different.
# This can be done with the function add_bias_to_traits.
example_man_traits <- manage_traits(example_genus_traits)
example_man_traits <- average_traits(example_man_traits, type = "C")

add_bias_to_traits(example_man_traits, fuzzy = FALSE)

# We can control the amount of bias with:
add_bias_to_traits(example_man_traits, fuzzy = FALSE, SD = 1)



#######################################
### 7. USING CUSTOM TRAITS DATASETS ###
#######################################

#############################
# a. North American example #
#############################

# This example uses aquatic insect traits from North America.
# Traits information have been retrieved and adapted from:
# Poff et al. (2006).Functional trait niches of North American lotic insects: traits-based ecological applications
# in light of phylogenetic relationships. Journal of the North American Benthological Society, 25(4), 730-755.
# Example Excel file available as SM3.

# Load traits data
poff_et_al_2006_traits <- as.data.frame(read_excel("./SM3_Poff_et_al 2006_traits.xlsx", sheet = "current"))

# Remove Leading/Trailing Whitespace.
poff_et_al_2006_traits$Taxa <- trimws(poff_et_al_2006_traits$Taxa)

# Get taxonomic information from GBIF.
poff_et_al_2006_gbif_tree <- get_gbif_taxa_tree(poff_et_al_2006_traits$Taxa)

# Create the taxonomic tree.
poff_et_al_2006_gbif_bio_tree <- ref_from_tree(poff_et_al_2006_gbif_tree$taxonomy) 

# Create a fake taxonomic dataset for illustrating purposes.
# We will select 10 taxa while generating abundances for 2 sites.
set.seed(1618)

macro_data <- data.frame(Taxa = sample(poff_et_al_2006_traits$Taxa, 10),
                         site_1 = sample(0:10, 10, replace = TRUE),
                         site_2 = sample(0:10, 10, replace = TRUE))

# Add some zeroes to increase heterogeneity.
macro_data[sample(c(TRUE, FALSE), 10, replace =TRUE, prob = c(0.4, 0.6) ), "site_1"] <- 0
macro_data[sample(c(TRUE, FALSE), 10, replace =TRUE, prob = c(0.2, 0.8) ), "site_2"] <- 0

# Import data into biomonitoR format with the function as_biomonitor.
macro_data_asb <- as_biomonitor(macro_data, dfref = poff_et_al_2006_gbif_bio_tree)

# Make a plot.
plot(macro_data_asb, child = "Genus")

# Use aggregate taxa for allow further calculations in biomonitoR.
macro_data_agr <- aggregate_taxa(macro_data_asb)

# Calculate genus and family richness.
richness(macro_data_agr, tax_lev = "Genus")
richness(macro_data_agr, tax_lev = "Family")

# Try with traits.
poff_et_al_2006_traits_family <- assign_traits(macro_data_agr,trait_db = poff_et_al_2006_traits,
                                               dfref = poff_et_al_2006_gbif_bio_tree,
                                               tax_lev = "Family")

head(poff_et_al_2006_traits_family)


### consider Poff et al. (2006) traits as continuous ###
# Average traits as continuous. Probably not the best choice because these traits
# are categorical and not continuous.
poff_et_al_2006_traits_family_av  <- average_traits(poff_et_al_2006_traits_family, type = "C")

# Functional richness.
f_rich(macro_data_agr, trait_db = poff_et_al_2006_traits_family_av, type = "C", distance = "euclidean", tax_lev = "Family")


# With custom distance.
rownames(poff_et_al_2006_traits_family_av) <- poff_et_al_2006_traits_family_av[, "Taxa"]
poff_et_al_2006_traits_dist <- dist(scale(poff_et_al_2006_traits_family_av[,-1]))

# Functional richness.
f_rich(macro_data_agr, trait_db = poff_et_al_2006_traits_dist, tax_lev = "Family")

### consider Poff et al. (2006) traits as categorical #
# We will use the Gower distance, implemented in the gower.dist function within the package StatMatch.
# We will also sampling randomly traits instead of calculate the average traits.
# Random sampling is done because traits are assigned at family level and for 
# each family we can have multiple genus-level traits.


# Given that we will randomly sampling traits, we will perform 99 iterations to
# see how much variability we get.

# Initialize the matrix where we will store the results.
results <- matrix(NA, ncol = 2, nrow = 99)

# Set variable names.
colnames(results) <- c("site_1", "site_2")


# Some traits are ordered, we need to tell this to R. We will do it trough a loop.
ordered_factors <- c("Volt", "Life", "Disp", "Crwl", "Swim", "Size", "Rheo", "Ther")

for(i in 6:ncol(poff_et_al_2006_traits_family)){
  # get the column name to check if it needs to be set as ordered
  cn <- colnames(poff_et_al_2006_traits_family)[i]
  
  if(any(cn %in% ordered_factors)){
    poff_et_al_2006_traits_family[, cn] <- factor(poff_et_al_2006_traits_family[, cn], ordered = TRUE,
                                                  levels = 1:length(unique(poff_et_al_2006_traits_family[, cn])) )
  } else{
    poff_et_al_2006_traits_family[, cn] <- factor(poff_et_al_2006_traits_family[, cn], ordered = FALSE,
                                                  levels = 1:length(unique(poff_et_al_2006_traits_family[, cn])) )
  }
}


# Calculate functional richness for each random sample. In this way we can 
# investigate the effect of within family heterogeneity of traits on functional richness.
for(i in 1:99){
  # resampled traits
  temp_traits <- sample_traits(poff_et_al_2006_traits_family)
  
  # calculate gower distance
  poff_et_al_2006_traits_gw_dist <- gower.dist(temp_traits[,-1])
  colnames(poff_et_al_2006_traits_gw_dist) <- rownames(poff_et_al_2006_traits_gw_dist) <- temp_traits[,1]
  
  # to avoid problems with the class returned by daisy
  poff_et_al_2006_traits_gw_dist <- as.dist(poff_et_al_2006_traits_gw_dist)
  
  # functional richness, with lingoes correction to make the distance matrix euclidean
  results[i,] <- f_rich(macro_data_agr, trait_db = poff_et_al_2006_traits_gw_dist, tax_lev = "Family", correction = "lingoes")
  
}

# Inspect the results. There is a lot of variability, probably family level is not a good choice here.
summary(results)



###############
# b. DISPERSE #
###############

# Example on dispersal traits of aquatic macroinvertebrates according to
# Sarremejane et al. (2020).
# DISPERSE, a trait database to assess the dispersal potential of European aquatic macroinvertebrates. 
# Scientific data, 7(1), 1-9.

# Traits retrieved from:
# https://springernature.figshare.com/collections/DISPERSE_A_trait_database_to_assess_the_dispersal_potential_of_aquatic_macroinvertebrates/5000633

# Download the disperse traits
disperse_traits <- read.csv(url("https://springernature.figshare.com/ndownloader/files/24964340"), h = TRUE)

# Remove unnecessary columns such as Reference list, Group, Family and Synonyms.
disperse_traits <- disperse_traits [, !grepl("Group|Family|Synonyms", colnames(disperse_traits))]

# Change the first column from Genus to Taxa name for compatibility with biomonitoR.
colnames(disperse_traits)[1] <- "Taxa"

# Setting column block, number of modalities for each trait. I did it manually, but it can be done in a more clever way
# if needed.
col_b <- c(7, 2, 3, 4, 4, 8, 5, 4, 3)

# Transform traits to percentages with the function prep.fuzzy of the ade4 package.
disperse_traits <- data.frame(disperse_traits[, "Taxa", drop = FALSE], prep.fuzzy(disperse_traits[,-1], col.blocks = col_b))

# Assign traits to the default mi_prin.
assign_traits(data_agg, trait_db = disperse_traits)
